package ac.ed.lurg.country;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.utils.StringTabularReader;

public class SubsidyRateManager {
	protected CompositeCountryManager compositeCountryManager;
	private StringTabularReader tabularReader;

	public SubsidyRateManager(CompositeCountryManager compositeCountryManager) {
		this.compositeCountryManager = compositeCountryManager;
		tabularReader = new StringTabularReader(",", new String[]{"country", "crop", "rate"});
		tabularReader.read(ModelConfig.SUBSIDY_RATE_FILE);
	}

	public Map<CropType, Double> getSubsidyRates(CompositeCountry cc) {

		Map<CropType, Double> subsidyRates = new HashMap<CropType, Double>();
		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			Map<CropType, Double> singleCMap = getSubsidyRates(c);

			for (CropType crop : CropType.getAllItems()) {
				Double newRateD = singleCMap.get(crop);
				Double prevRateD = subsidyRates.get(crop);

				// just takes max which isn't right. Should instead weight by production or something else
				double newRate = Math.max(getDoubleOrZero(newRateD), getDoubleOrZero(prevRateD));
				subsidyRates.put(crop, newRate);
			}
		}
		return subsidyRates;
	}

	private double getDoubleOrZero(Double d) {
		return (d==null ? 0.0 : d.doubleValue());
	}

	private Map<CropType, Double> getSubsidyRates(SingleCountry c) {
		Map<CropType, Double> rates = new HashMap<CropType, Double>();

		for (CropType crop : CropType.getAllItems()) {
			Map<String, String> queryMap = new HashMap<String, String>();
			queryMap.put("country", c.getCountryName());
			queryMap.put("crop", crop.getGamsName());
			try {
				double subsidy = 0 ;
				List<Map<String, String>> rows = tabularReader.query(queryMap);
				if (rows.size() == 1) {
					String subsidyS = rows.get(0).get("rate");
					subsidy = Double.valueOf(subsidyS);
				}
				rates.put(crop, subsidy);
			}
			catch (Exception e) {
					LogWriter.println("Problem finding cereal demand: " + crop.getFaoName() + ", " + c.getCountryName());
			}
		}
		return rates;
	}
}
