package ac.ed.lurg.country.crafty;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import ac.ed.lurg.country.AbstractCountryAgent;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.GlobalPrice;
import ac.ed.lurg.demand.AbstractDemandManager;
import ac.ed.lurg.landuse.CropUsageData;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;

/**
 * Country agent that is interface over data transferred to and from CRAFTY.
 * Need to configure country mapping to that there is
 * */
public class CraftyCountryAgent extends AbstractCountryAgent {

	private Map<CropType, CropUsageData> cropUsageData;

	public CraftyCountryAgent(AbstractDemandManager demandManager,CompositeCountry country, Map<CropType, Double> tradeBarriers) {
		super(demandManager, country, tradeBarriers);
	}

	public Map<CropType, CropUsageData> getCropUsageData() {
		return cropUsageData;
	}

	public void updateProduction(Map<CropType, Double> cropProduction, Map<CropType, GlobalPrice> worldPrices, GlobalPrice carbonPrice, GlobalPrice timberPrice) {
		calculateCountryPricesAndDemand(worldPrices, carbonPrice, timberPrice, false);

		cropUsageData = new HashMap<CropType, CropUsageData>();
		for (Entry<CropType, Double> entry : cropProduction.entrySet()) {
			CropUsageData cropusage = new CropUsageData(entry.getValue());
			cropUsageData.put(entry.getKey(), cropusage);
		}

		updateNetImportsFromProdAndDemand(currentProjectedDemand, currentMinDemandFract, cropUsageData);
	}

	@Override
 	protected double getCommPriceFromCropPrice(CommodityType commodity) {
 		double commPricePlum = 0;
 		Map<CropType, Double> demandFract = baseDemandFact.get(commodity);
 
 		for (CropType crop : commodity.getCropTypes()) {
 			commPricePlum += currentCountryPrices.get(crop).getImportPrice() * demandFract.get(crop);  // weight price by base demand of each cereal crop
 		}
 		return commPricePlum;
 	}
	
	public double getNetCarbonFlux() {
		return 0;
	}
	
	public double getTimberProduction() {
		return 0;
	}
}