package ac.ed.lurg.country.gams;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CountryPrice;
import ac.ed.lurg.country.TradeConstraint;
import ac.ed.lurg.landuse.CropUsageData;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.types.CropType;

public class GamsCountryInput {

	private CompositeCountry country; // not really required but useful for debugging
	private Map<CommodityType, Double> projectedDemand;
	private Map<CropType, TradeConstraint> tradeConstraints;
	private Map<CropType, CountryPrice> countryPrices;
	private Map<CropType, CropUsageData> previousCropUsageData;
	private Map<CommodityType, Map<CropType, Double>> minDemandFractions;
	private Map<CropType, Double> subsidyRates;
	private CountryPrice carbonPrice;
	private TradeConstraint carbonTradeConstraint;
	private CountryPrice timberPrice;
	private TradeConstraint timberTradeConstraint;

	public GamsCountryInput(CompositeCountry country, Map<CommodityType, Double> projectedDemand, Map<CropType, CountryPrice> countryPrices, 
			Map<CropType, TradeConstraint> importConstraints, Map<CropType, CropUsageData> previousCropUsageData, 
			Map<CommodityType, Map<CropType, Double>> minDemandFracts, Map<CropType, Double> subsidyRates,
			CountryPrice carbonPrice, TradeConstraint carbonTradeConstraint, CountryPrice timberPrice, TradeConstraint timberTradeConstraint) {
		super();
		this.country = country;
		this.projectedDemand = projectedDemand;
		this.tradeConstraints = importConstraints;
		this.countryPrices = countryPrices;
		this.previousCropUsageData = previousCropUsageData;
		this.minDemandFractions = minDemandFracts;
		this.subsidyRates = subsidyRates;
		this.carbonPrice = carbonPrice;
		this.carbonTradeConstraint = carbonTradeConstraint;
		this.timberPrice= timberPrice;
		this.timberTradeConstraint = timberTradeConstraint;
	}
		
	public CompositeCountry getCountry() { 
		return country;
	}
	
	public Map<CommodityType, Double> getProjectedDemand() {
		return projectedDemand;
	}

	public Map<CropType, CountryPrice> getCountryPrices() {
		return countryPrices;
	}
	
	public Map<CropType, TradeConstraint> getTradeConstraints() {
		return tradeConstraints;
	}
	
	public Map<CropType, CropUsageData> getPreviousCropUsageData() {
		return previousCropUsageData;
	}

	public double getMeatEfficiency() {
		
		
		return ModelConfig.MEAT_EFFICIENCY;  // this is already handled by the feed conversion efficiency for each animal product
	}

	public Map<CropType, Double> getMinTrade() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, TradeConstraint> entry : tradeConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMinConstraint());
		}
		return netImport;
	}

	public Map<CropType, Double> getMaxTrade() {
		Map<CropType, Double> netImport = new HashMap<CropType, Double>();
		for (Map.Entry<CropType, TradeConstraint> entry : tradeConstraints.entrySet()) {
			netImport.put(entry.getKey(), entry.getValue().getMaxConstraint());
		}
		return netImport;
	}
	
	public Map<CommodityType, Map<CropType, Double>> getMinDemandFractions() {
		return minDemandFractions;
	}
	
	public Map<CropType, Double> getSubsidyRates() {
		return subsidyRates;
	}
	
	public CountryPrice getCarbonPrice() {
		return carbonPrice;
	}
	
	public TradeConstraint getCarbonTradeConstraint() {
		return carbonTradeConstraint;
	}

	public CountryPrice getTimberPrice() {
		return timberPrice;
	}

	public TradeConstraint getTimberTradeConstraint() {
		return timberTradeConstraint;
	}

	public void setTradeConstraints(Map<CropType, TradeConstraint> tradeConstraints) {
		this.tradeConstraints = tradeConstraints;
	}
}