package ac.ed.lurg.country.gams;

import java.util.Map;

import ac.ed.lurg.Timestep;
import ac.ed.lurg.landuse.LandUseItem;
import ac.ed.lurg.landuse.WoodYieldItem;
import ac.ed.lurg.types.LandCoverType;
import ac.ed.lurg.utils.DoubleMap;
import ac.ed.lurg.landuse.CarbonFluxItem;
import ac.ed.lurg.landuse.IrrigationItem;
import ac.ed.lurg.yield.YieldRaster;
import ac.sac.raster.RasterSet;

public class GamsRasterInput {
	
	private Timestep timestep;
	private YieldRaster yields;
	private RasterSet<LandUseItem> previousLandsUses;
	private RasterSet<IrrigationItem> irrigationCost;
	private RasterSet<CarbonFluxItem> carbonFluxes;
	private RasterSet<WoodYieldItem> woodYields;
	private DoubleMap<Integer, LandCoverType, Double> minimumLandCover;
	private DoubleMap<Integer, LandCoverType, Double> timberYield;
	private Map<Integer, Double> naturalWoodYieldFactor;
	private DoubleMap<LandCoverType, LandCoverType, Double> conversionCosts;
	private GamsCountryInput countryInput;

	public GamsRasterInput(Timestep timestep, YieldRaster yields, RasterSet<LandUseItem> previousLandsUses, RasterSet<IrrigationItem> irrigationCost, 
			RasterSet<CarbonFluxItem> carbonFluxes, RasterSet<WoodYieldItem> woodYields, DoubleMap<Integer, LandCoverType, Double> minimumLandCover,
			DoubleMap<Integer, LandCoverType, Double> timberYield, Map<Integer, Double> naturalWoodYieldFactor, 
			DoubleMap<LandCoverType, LandCoverType, Double> conversionCosts, GamsCountryInput countryInput) {
		super();
		this.timestep = timestep;
		this.yields = yields;
		this.previousLandsUses = previousLandsUses;
		this.irrigationCost = irrigationCost;
		this.carbonFluxes = carbonFluxes;
		this.woodYields = woodYields;
		this.minimumLandCover = minimumLandCover;
		this.timberYield = timberYield;
		this.naturalWoodYieldFactor = naturalWoodYieldFactor;
		this.conversionCosts = conversionCosts;
		this.countryInput = countryInput;
	}

	public YieldRaster getYields() {
		return yields;
	}

	public RasterSet<LandUseItem> getPreviousLandUses() {
		return previousLandsUses;
	}
	
	public RasterSet<IrrigationItem> getIrrigationData() {
		return irrigationCost;
	}
	
	public RasterSet<CarbonFluxItem> getCarbonFluxes() {
		return carbonFluxes;
	}
	
	public RasterSet<WoodYieldItem> getWoodYields() {
		return woodYields;
	}
	
	public DoubleMap<LandCoverType, LandCoverType, Double> getConversionCosts() {
		return conversionCosts;
	}
	
	public DoubleMap<Integer, LandCoverType, Double> getMinimumLandCover() {
		return minimumLandCover;
	}
	
	public DoubleMap<Integer, LandCoverType, Double> getTimberYield() {
		return timberYield;
	}
	
	public Map<Integer, Double> getNaturalWoodYieldFactor() {
		return naturalWoodYieldFactor;
	}

	public GamsCountryInput getCountryInput() {
		return countryInput;
	}
	
	public Timestep getTimestep() {
		return timestep;
	}
}
