package ac.ed.lurg.country.gams;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;

public class TauCalculationManager { 
	// to change Tau parameters of the elastic demand diet curves which changes the diet at high income country (saturation level)
	// Tau is changed for each commodity (unique for all countries)
	// Change from START_YEAR from the "normal" coefficients from Gouel & Guimbard 2019 based on historical data (higher income country trend to have a rich in sugar, fat and animal protein diet 
	// To a diet specified and created in the plumv2>Data>Tau_diet.csv file (in this case an Healthy diet from EAT LANCET article)

	private Map<CommodityType, Double> shiftTauMap = new HashMap<CommodityType, Double>();

	private static final int ITEM_COL = 0; 
	private static final int SHIFT_TAU_COL = 1; 

	
	private static TauCalculationManager tcm;
    public static TauCalculationManager getInstance() { 
        if (tcm == null) 
        	tcm = new TauCalculationManager(); 
  
        return tcm; 
    }
    
	private TauCalculationManager() {
		readTauFromFile(); 
	}

	public double getFinalTau(int year, double initialTau, CommodityType commodity) {
		double shiftTau = shiftTauMap.get(commodity);
		double tau;

		if(year > ModelConfig.DIET_CHANGE_START_YEAR && year < ModelConfig.DIET_CHANGE_END_YEAR)
			tau = initialTau + ((year - ModelConfig.DIET_CHANGE_START_YEAR) * (shiftTau - initialTau) / (ModelConfig.DIET_CHANGE_END_YEAR- ModelConfig.DIET_CHANGE_START_YEAR));  //  interpolation
		else if (year >= ModelConfig.DIET_CHANGE_END_YEAR)
			tau = shiftTau; 
		else
			tau = initialTau;
		
		return tau;
	}

	//Read File for Tau and tau diet change
	private void readTauFromFile() {
		String filename = ModelConfig.TARGET_DIET_FILE;
		try {
			BufferedReader fitReader = new BufferedReader(new FileReader(filename)); 
			String line, gamsName;
			Double shiftTau;
			fitReader.readLine(); // read header

			while ((line=fitReader.readLine()) != null) {
				String[] tokens = line.split(",");

				if (tokens.length < 2)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);

				gamsName = tokens[ITEM_COL];
				shiftTau = Double.parseDouble(tokens[SHIFT_TAU_COL]);

				CommodityType commodity = CommodityType.getForGamsName(gamsName);

				shiftTauMap.put(commodity,shiftTau);
			} 
			fitReader.close(); 

		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading Tau file");
			LogWriter.print(e);
		}
		LogWriter.println("Processed " + filename);
	}
}