package ac.ed.lurg.demand;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.utils.Interpolator;
import ac.ed.lurg.utils.LogWriter;

public class CarbonDemandManager {
	private Map<Integer, Double> globalCarbonDemand; // global demand for carbon sequestration
	private static final int YEAR_COL = 0;
	private static final int DEMAND_COL = 1; 
	
	public CarbonDemandManager() {
		readCarbonDemandData();
	}
	
	public void readCarbonDemandData() {
		
		Map<Integer, Double> data = new HashMap<Integer, Double>();
		
		String filename = ModelConfig.CARBON_DEMAND_FILE;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename)); 
			String line;
			Integer year;
			Double demand;
			reader.readLine(); // read header

			while ((line=reader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 2)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				year = Integer.valueOf(tokens[YEAR_COL]);
				demand = Double.valueOf(tokens[DEMAND_COL]);
				
				data.put(year,  demand);
			} 
			reader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading carbon demand data");
			LogWriter.print(e);
		}
		globalCarbonDemand = data;
		LogWriter.println("Processed " + filename);
	}
	
	public double getGlobalCarbonDemand(int year) {
		int downYear = (year/5) * 5;
		int upYear = downYear + 5;
		Double lowerD = globalCarbonDemand.get(downYear);
		Double upperD = globalCarbonDemand.get(upYear);
		double factor = ((double)(year - downYear)) / (upYear - downYear);
		Double d = Interpolator.interpolate(lowerD, upperD, factor);
		return d;
	}
}
