package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.utils.StringTabularReader;

public class PriceMarkUps {
	private StringTabularReader intialConsumerPrices;
	private Map<CountryCommodityKey, Double> priceMarkupFactors = new HashMap<CountryCommodityKey, Double>();

	public PriceMarkUps() {
		// could read priceMarkupFactors from file here in some cases
		
		intialConsumerPrices = new StringTabularReader(",", new String[]{"Item", "country", "price2010"});
		intialConsumerPrices.read(ModelConfig.INITIAL_CONSUMER_PRICE_FILE);
	}
	
	public double markedupPrice(SingleCountry c, CommodityType commodity, double producerPrice) {
		CountryCommodityKey cc = new CountryCommodityKey(c, commodity);
		Double priceMarkupFactor = priceMarkupFactors.get(cc);

		if (priceMarkupFactor == null) {
			double consumerPrice = getInitialConsumerPrice(c, commodity);
			priceMarkupFactor = consumerPrice / producerPrice;
			LogWriter.println("priceMarkupFactor for " + c + ", " + commodity.getGamsName() + " is " + priceMarkupFactor);
			priceMarkupFactors.put(cc, priceMarkupFactor);
		}
		
		double adjustedPrice = producerPrice * priceMarkupFactor;
		return adjustedPrice;
	}
	
	private double getInitialConsumerPrice(SingleCountry country, CommodityType commodity) {
		
		double initialPrice;
		Map<String, String> queryMap = new HashMap<String, String>();
		queryMap.put("country", country.getCountryName());
		queryMap.put("Item", commodity.getGamsName());

		try {
			Map<String, String> row = intialConsumerPrices.querySingle(queryMap);
			String priceS = row.get("price2010");
			initialPrice = Double.valueOf(priceS);
		} catch (Exception e) {
			LogWriter.println("Problem finding initial consumer price " + commodity.getGamsName() + ", " + commodity.getGamsName() + ", " + country.getCountryName());
			initialPrice=commodity.getDefaultInitialPrice();
		}

		return initialPrice;
	}
	
	private class CountryCommodityKey {
		private SingleCountry country;
		private CommodityType commodity;
		CountryCommodityKey (SingleCountry country, CommodityType commodity) {
			this.country = country;
			this.commodity = commodity;
		}
		@Override
		public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + getOuterType().hashCode();
			result = prime * result + ((commodity == null) ? 0 : commodity.hashCode());
			result = prime * result + ((country == null) ? 0 : country.hashCode());
			return result;
		}
		@Override
		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			CountryCommodityKey other = (CountryCommodityKey) obj;
			if (!getOuterType().equals(other.getOuterType()))
				return false;
			if (commodity != other.commodity)
				return false;
			if (country == null) {
				if (other.country != null)
					return false;
			} else if (!country.equals(other.country))
				return false;
			return true;
		}
		private PriceMarkUps getOuterType() {
			return PriceMarkUps.this;
		}
	}
}
