package ac.ed.lurg.demand;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.utils.Interpolator;
import ac.ed.lurg.utils.LogWriter;

public class TimberDemandManager {
	private Map<Integer, Double> globalTimberDemand; // global demand for timber
	private static final int YEAR_COL = 0;
	private static final int DEMAND_COL = 1; 
	
	public TimberDemandManager() {
		readTimberDemandData();
	}
	
	public void readTimberDemandData() {
		
		Map<Integer, Double> data = new HashMap<Integer, Double>();
		
		String filename = ModelConfig.TIMBER_DEMAND_FILE;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename)); 
			String line;
			Integer year;
			Double demand;
			reader.readLine(); // read header

			while ((line=reader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 2)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				year = Integer.valueOf(tokens[YEAR_COL]);
				demand = Double.valueOf(tokens[DEMAND_COL]);
				
				data.put(year,  demand);
			} 
			reader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading timber demand data");
			LogWriter.print(e);
		}
		globalTimberDemand = data;
		LogWriter.println("Processed " + filename);
	}
	
	public double getGlobalTimberDemand(int year) {
		int downYear = (year/5) * 5;
		int upYear = downYear + 5;
		Double lowerD = globalTimberDemand.get(downYear);
		Double upperD = globalTimberDemand.get(upYear);
		double factor = ((double)(year - downYear)) / (upYear - downYear);
		Double d = Interpolator.interpolate(lowerD, upperD, factor);
		return d;
	}
}
