package ac.ed.lurg.landuse;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.types.LandCoverType;
import ac.ed.lurg.utils.DoubleMap;
import ac.ed.lurg.utils.LogWriter;

public class ConversionCostReader {
	
	private static final int FROM_COL = 0;
	private static final int TO_COL = 1;
	private static final int COST_COL = 2;
	
	public DoubleMap<LandCoverType, LandCoverType, Double> read() {
		DoubleMap<LandCoverType, LandCoverType, Double> conversionCosts = new DoubleMap<LandCoverType, LandCoverType, Double>();
		String filename = ModelConfig.CONVERSION_COST_FILE;
		try {
			BufferedReader reader = new BufferedReader(new FileReader(filename)); 
			String line, fromName, toName;
			Double cost;
			reader.readLine(); // read header

			while ((line=reader.readLine()) != null) {
				String[] tokens = line.split(",");
				
				if (tokens.length < 3)
					LogWriter.printlnError("Too few columns in " + filename + ", " + line);
				
				fromName = tokens[FROM_COL];
				toName = tokens[TO_COL];
				cost = Double.parseDouble(tokens[COST_COL]);
				
				LandCoverType fromLC = LandCoverType.getForName(fromName);
				LandCoverType toLC = LandCoverType.getForName(toName);
				
				conversionCosts.put(fromLC, toLC, cost);
			} 
			reader.close(); 
		
		} catch (IOException e) {
			LogWriter.printlnError("Failed in reading conversion costs file");
			LogWriter.print(e);
		}
		LogWriter.println("Processed " + filename);
		
		return conversionCosts;
	}

}
