package ac.ed.lurg.landuse;

import java.util.Map;

import ac.ed.lurg.types.LandCoverType;
import ac.sac.raster.AbstractTabularRasterReader;
import ac.sac.raster.RasterKey;
import ac.sac.raster.RasterSet;

public class WoodYieldReader extends AbstractTabularRasterReader<WoodYieldItem> {
	
	private static final int MIN_COLS = 27;
	private static final double conversionFactor = 10.0; // convert kgC/m2 to tC/ha
	
	public WoodYieldReader(RasterSet<WoodYieldItem> woodYield) {
		super("[ |\t]+", MIN_COLS, woodYield);
	}

	protected void setData(RasterKey key, WoodYieldItem item, Map<String, Double> rowValues) {
		item.setWoodYield(LandCoverType.CROPLAND, LandCoverType.NATURAL,    getValueForCol(rowValues, "crop_to_ntrl") * conversionFactor);
		item.setWoodYield(LandCoverType.CROPLAND, LandCoverType.TIMBER_FOREST,    getValueForCol(rowValues, "crop_to_forT") * conversionFactor);
		item.setWoodYield(LandCoverType.CROPLAND, LandCoverType.CARBON_FOREST,    getValueForCol(rowValues, "crop_to_forC") * conversionFactor);
		item.setWoodYield(LandCoverType.CROPLAND, LandCoverType.PASTURE,          getValueForCol(rowValues, "crop_to_past") * conversionFactor);
		item.setWoodYield(LandCoverType.CROPLAND, LandCoverType.CROPLAND,         getValueForCol(rowValues, "crop_to_crop") * conversionFactor);
		                                                                                                                       
		item.setWoodYield(LandCoverType.PASTURE, LandCoverType.NATURAL,    getValueForCol(rowValues, "past_to_ntrl") * conversionFactor);
		item.setWoodYield(LandCoverType.PASTURE, LandCoverType.TIMBER_FOREST,    getValueForCol(rowValues, "past_to_forT") * conversionFactor);
		item.setWoodYield(LandCoverType.PASTURE, LandCoverType.CARBON_FOREST,    getValueForCol(rowValues, "past_to_forC") * conversionFactor);
		item.setWoodYield(LandCoverType.PASTURE, LandCoverType.CROPLAND,         getValueForCol(rowValues, "past_to_crop") * conversionFactor);
		item.setWoodYield(LandCoverType.PASTURE, LandCoverType.PASTURE,          getValueForCol(rowValues, "past_to_past") * conversionFactor);
		
		item.setWoodYield(LandCoverType.NATURAL, LandCoverType.PASTURE,  	   	   getValueForCol(rowValues, "ntrl_to_past") * conversionFactor);
		item.setWoodYield(LandCoverType.NATURAL, LandCoverType.TIMBER_FOREST,    getValueForCol(rowValues, "ntrl_to_forT") * conversionFactor);
		item.setWoodYield(LandCoverType.NATURAL, LandCoverType.CARBON_FOREST,    getValueForCol(rowValues, "ntrl_to_forC") * conversionFactor);
		item.setWoodYield(LandCoverType.NATURAL, LandCoverType.CROPLAND,         getValueForCol(rowValues, "ntrl_to_crop") * conversionFactor);
		item.setWoodYield(LandCoverType.NATURAL, LandCoverType.NATURAL,    getValueForCol(rowValues, "ntrl_to_ntrl") * conversionFactor);	
		
		item.setWoodYield(LandCoverType.TIMBER_FOREST, LandCoverType.PASTURE,  	    getValueForCol(rowValues, "forT_to_past") * conversionFactor);
		item.setWoodYield(LandCoverType.TIMBER_FOREST, LandCoverType.NATURAL,  	getValueForCol(rowValues, "forT_to_ntrl") * conversionFactor);
		item.setWoodYield(LandCoverType.TIMBER_FOREST, LandCoverType.CARBON_FOREST,    getValueForCol(rowValues, "forT_to_forC") * conversionFactor);
		item.setWoodYield(LandCoverType.TIMBER_FOREST, LandCoverType.CROPLAND,         getValueForCol(rowValues, "forT_to_crop") * conversionFactor);
		item.setWoodYield(LandCoverType.TIMBER_FOREST, LandCoverType.TIMBER_FOREST,    getValueForCol(rowValues, "forT_to_forT") * conversionFactor);
		                                                                                                                                
		item.setWoodYield(LandCoverType.CARBON_FOREST, LandCoverType.PASTURE,  	    getValueForCol(rowValues, "forC_to_past") * conversionFactor);
		item.setWoodYield(LandCoverType.CARBON_FOREST, LandCoverType.NATURAL,  	getValueForCol(rowValues, "forC_to_ntrl") * conversionFactor);
		item.setWoodYield(LandCoverType.CARBON_FOREST, LandCoverType.TIMBER_FOREST,    getValueForCol(rowValues, "forC_to_forT") * conversionFactor);
		item.setWoodYield(LandCoverType.CARBON_FOREST, LandCoverType.CROPLAND,         getValueForCol(rowValues, "forC_to_crop") * conversionFactor);
		item.setWoodYield(LandCoverType.CARBON_FOREST, LandCoverType.CARBON_FOREST,    getValueForCol(rowValues, "forC_to_forC") * conversionFactor);                                                                          
	}
}
