package ac.ed.lurg.shock;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.Timestep;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;
import ac.ed.lurg.utils.StringTabularReader;

public class ExportRestrictionManager {
	private StringTabularReader exportShockReader;

	public ExportRestrictionManager() {

		String shockFile = ModelConfig.EXPORT_RESTRICTIONS_FILE;
		exportShockReader = new StringTabularReader(",", new String[]{"year", "country", "crop", "shockFactor"});

		if (new File(shockFile).isFile()) 
			exportShockReader.read(shockFile);
		else
			LogWriter.println("Can't find a export shock file (" +shockFile + "), so will not apply export restriction");
	}
	
	public Map<CropType, Double> getShocksForCountry(Timestep timestep, CompositeCountry cc) {
		Map<CropType, Double> shocks = new HashMap<CropType, Double>();

		Map<String, String> queryMap = new HashMap<String, String>();
		queryMap.put("year", Integer.toString(timestep.getYear()));
		queryMap.put("country", cc.getName());
		
		List<Map<String, String>> rows = exportShockReader.query(queryMap);
		for (Map<String, String> row : rows) {
		String cropTypeS = row.get("crop");
		String shockFactorS = row.get("shockFactor");
		CropType crop = CropType.getForFaoName(cropTypeS);
		Double shockFactor = Double.valueOf(shockFactorS);

		LogWriter.println("Got export shock at " + timestep + ": " + cc.getName() + " " + cropTypeS + " " + shockFactorS);
		
		shocks.put(crop, shockFactor);
		}
		return shocks;
	}
}