package ac.ed.lurg.types;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import ac.ed.lurg.ModelConfig;

public enum CommodityType {

	//processedFull$cb_fs[is.na(fcr) & Country=="World" & Year == 2010, list(energy=sum(energy_kcal_pc*365/1000000*country_data[Country=="World" & Year == 2010]$population, na.rm=TRUE), food=sum(food)), by="plumDemandItem"][, list(plumDemandItem, energy/food)]
	//processedFull$cb_fs[!is.na(fcr) & Country=="World" & Year == 2010, list(energy=sum(energy_kcal_pc*365/1000000*country_data[Country=="World" & Year == 2010]$population, na.rm=TRUE), food=sum(food*fcr)), by="plumDemandItem"][, list(plumDemandItem, energy/food)]
	CEREALS("Cereals", "cereals", false, new CropType[]{CropType.WHEAT, CropType.MAIZE, CropType.RICE}, new Double[]{0.385, 0.171, 0.444}, ModelConfig.CEREALS_SUB_PROPORTION, ModelConfig.INITAL_DEMAND_PRICE_CEREALS, 409.0),
	OILCROPSPULSES("OilcropsPulses", "oilcropspulses", false, new CropType[]{CropType.OILCROPS, CropType.PULSES}, new Double[]{0.815, 0.185}, ModelConfig.OILCROPSPULSES_SUB_PROPORTION, ModelConfig.INITAL_DEMAND_PRICE_OILCROPS_PULSES, 23.0),
	STARCHY_ROOTS("Starchy Roots", "starchyRoots", false, new CropType[]{CropType.STARCHY_ROOTS}, new Double[]{1.0}, ModelConfig.STARCHY_ROOTS_SUB_PROPORTION, ModelConfig.INITAL_DEMAND_PRICE_STARCHYROOTS, 14.0),
	MONOGASTRICS("Monogastrics", "monogastrics", true, new CropType[]{CropType.MONOGASTRICS}, new Double[]{1.0}, Double.NaN, ModelConfig.INITAL_DEMAND_PRICE_MONOGASTRICS, 3.0),
	RUMINANTS("Ruminants", "ruminants", true, new CropType[]{CropType.RUMINANTS}, new Double[]{1.0}, Double.NaN, ModelConfig.INITAL_DEMAND_PRICE_RUMINANTS, 20.0),
	FRUITVEG("FruitVeg", "fruitveg", false, new CropType[]{CropType.FRUITVEG}, new Double[]{1.0}, ModelConfig.FRUITVEG_SUB_PROPORTION, ModelConfig.INITAL_DEMAND_PRICE_FRUITVEG, 21.0),
	SUGAR("Sugar", "sugar", false, new CropType[]{CropType.SUGAR}, new Double[]{1.0}, ModelConfig.SUGAR_SUB_PROPORTION, ModelConfig.INITAL_DEMAND_PRICE_SUGAR, 35.0),
	NONFOOD("Nonfood", "nonfood", false, new CropType[]{}, new Double[]{}, Double.NaN, Double.NaN, 0.0);

	private String faoName;
	private String gamsName;
	private boolean isAnimalProduct;
	private Collection<CropType> cropTypes;
	private double meatSubstitutionProportion;
	private double initialPrice;
	private double minFAOCalorieIntake;
	
	CommodityType (String faoName, String gamsName, boolean isAnimalProduct, CropType[] cropTypeMapping, Double[] defaultDemandFract, 
			double meatSubstitutionProportion, double initialPrice, double minFAOCalorieIntake) {
		this.faoName = faoName;
		this.gamsName = gamsName;
		this.isAnimalProduct = isAnimalProduct;
		this.cropTypes = Arrays.asList(cropTypeMapping);
		this.meatSubstitutionProportion = meatSubstitutionProportion;
		this.initialPrice = initialPrice;
		this.minFAOCalorieIntake = minFAOCalorieIntake;
	}
	
	private static final Map<String, CommodityType> faoNameCache = new HashMap<String, CommodityType>();
	private static final Map<String, CommodityType> gamsNameCache = new HashMap<String, CommodityType>();
	private static final Collection<CommodityType> foodsToConsider = new HashSet<CommodityType>();
	static {
		for (CommodityType c : values()) {
			faoNameCache.put(c.getFaoName(), c);
			gamsNameCache.put(c.getGamsName(), c);
			
			if (!c.equals(NONFOOD))
				foodsToConsider.add(c);
		}
	}

	public static CommodityType getForFaoName(String faoName) {
		CommodityType commodity = faoNameCache.get(faoName);

		if (commodity == null) 
			throw new RuntimeException("Can't find CommodityType for faoName: " + faoName);
		
		return commodity;
	}
	
	public static CommodityType getForGamsName(String gamsName) {
		return getForGamsName(gamsName, false);
	}
	
	public static CommodityType getForGamsName(String gamsName, boolean allowNulls) {
		CommodityType commodity = gamsNameCache.get(gamsName);

		if (commodity == null & !allowNulls) 
			throw new RuntimeException("Can't find CommodityType for gamsName: " + gamsName);
		
		return commodity;
	}
	
	public static Collection<CommodityType> getAllFoodItems() {
		return foodsToConsider;
	}

	public String getFaoName() {
		return faoName;
	}
	
	public String getGamsName() {
		return gamsName;
	}

	public boolean isAnimalProduct() {
		return isAnimalProduct;
	}
	
	public double getMeatSubstitutionProportion(){
		return meatSubstitutionProportion;
	}
	
	public Collection<CropType> getCropTypes(){
		return cropTypes;
	}
	
	public double getDefaultInitialPrice() {
		return initialPrice;
	}
	
	public double getMinFAOCalorieIntake() {
		return minFAOCalorieIntake;
	}
}