package ac.ed.lurg.types;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.utils.LogWriter;

public enum CropType {

	WHEAT("WheatBarleyOats", "wheat", true, false, 9.5, ModelConfig.INITAL_PRICE_WHEAT),
	MAIZE("MaizeMilletSorghum", "maize", true, false, 5.1, ModelConfig.INITAL_PRICE_MAIZE),
	RICE("Rice (Paddy Equivalent)", "rice", true, false, 8.3, ModelConfig.INITAL_PRICE_RICE),
	OILCROPS("Oilcrops", "oilcrops", true, false, 4.4, ModelConfig.INITAL_PRICE_OILCROPS),
	PULSES("Pulses", "pulses", true, false, 10.8, ModelConfig.INITAL_PRICE_PULSES),
	STARCHY_ROOTS("Starchy Roots", "starchyRoots", true, false, 14.3, ModelConfig.INITAL_PRICE_STARCHYROOTS),
	ENERGY_CROPS("Energy crops", "energycrops", true, false, 5, ModelConfig.INITAL_PRICE_ENERGYCROPS),
	SETASIDE("setaside", "setaside", false, false, 0, 0),
	MONOGASTRICS("Monogastrics", "monogastrics", true, true, 3.1, ModelConfig.INITAL_PRICE_MONOGASTRICS),
	RUMINANTS("Ruminants", "ruminants", true, true, 2.2, ModelConfig.INITAL_PRICE_RUMINANTS),
	FRUITVEG("FruitVeg", "fruitveg", true, false, 8.9, ModelConfig.INITAL_PRICE_FRUITVEG),
	SUGAR("Sugar", "sugar", true, false, 0.1, ModelConfig.INITAL_PRICE_SUGAR),
	PASTURE("pasture", "pasture", false, false, 0, 0);  // confusing having a land cover and a crop type.  Needed here for yields (but not used for cropland area fractions).

	private String faoName;
	private String gamsName;
	private boolean importedCrop;
	private boolean isMeat;
	private double seedAndWasteRate;
	private double initialPrice;
	
	private static Collection<CropType> importedCrops;

	private CropType (String faoName, String gamsName, boolean importedCrop, boolean isMeat, double seedAndWastePercent, double initialPrice) {
		this.faoName = faoName;
		this.gamsName = gamsName;
		this.importedCrop = importedCrop;
		this.isMeat = isMeat;
		this.seedAndWasteRate = seedAndWastePercent/100;
		this.initialPrice = initialPrice;
	}
	
	public static Collection<CropType> getCropsLessPasture() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (!c.equals(CropType.PASTURE) && !c.isMeat)
				comms.add(c);
		
		return comms;
	}

	public static synchronized Collection<CropType> getImportedTypes() {
		if (importedCrops == null) {
			importedCrops = new LinkedList<CropType>(); // what consistent order
		
			for (CropType c : values())
				if (c.importedCrop)
					importedCrops.add(c);
		}
		
		return importedCrops;
	}
	
	public static Collection<CropType> getNonMeatTypes() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (!c.isMeat)
				comms.add(c);
		
		return comms;
	}
	
	public static Collection<CropType> getMeatTypes() {
		Collection<CropType> comms = new HashSet<CropType>();
		
		for (CropType c : values())
			if (c.isMeat)
				comms.add(c);
		
		return comms;
	}


	private static final Map<String, CropType> faoNameCache = new HashMap<String, CropType>();
	private static final Map<String, CropType> gamsNameCache = new HashMap<String, CropType>();
    static {
        for (CropType c : values()) {
        	faoNameCache.put(c.getFaoName(), c);
        	gamsNameCache.put(c.getGamsName(), c);
       }
    }

	public static CropType getForFaoName(String faoName) {
		CropType crop = getFromCache(faoNameCache, faoName);
		if (crop == null) {
			throw new RuntimeException("Can't find CropType for faoName: " + faoName);
		}
		return crop;
	}
	
	public static CropType getForGamsName(String gamsName) {
		return getFromCache(gamsNameCache, gamsName);
	}
	
	private static CropType getFromCache(Map<String, CropType> cache, String name) {
		CropType type = cache.get(name);
		
		if (type == null)
			LogWriter.printlnError("Can't find Item for " + name);
		
		return type;
	}
	
	public static Collection<CropType> getAllItems() {
		return gamsNameCache.values();
	}

	public String getFaoName() {
		return faoName;
	}

	public String getGamsName() {
		return gamsName;
	}
	
	public boolean isImportedCrop() {
		return importedCrop;
	}

	public double getSeedAndWasteRate() {
		return seedAndWasteRate;
	}

	public double getInitialPrice() {
		return initialPrice;
	}
}
