package ac.ed.lurg.types;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import ac.ed.lurg.utils.LogWriter;

public enum LandCoverType {
	
	TIMBER_FOREST("timberForest", false, true, true, true),
	CARBON_FOREST("carbonForest", false, true, true, true),
	NATURAL ("natural", true, true, true, false),
	CROPLAND ("cropland", false, true, false, false),
	PASTURE ("pasture", false, true, false, false),
	BARREN ("barren", false, false, false, false),
	URBAN("urban", false, false, false, false);

	private String name;
	private boolean isProtectable;
	private boolean isConvertible;
	private boolean isNatural;
	private boolean isManagedForest;
	
	LandCoverType(String name, boolean isProtectable, boolean isConvertible, boolean isNatural, boolean isManagedForest) {
		this.name = name;
		this.isProtectable = isProtectable;
		this.isConvertible = isConvertible;
		this.isNatural = isNatural;
		this.isManagedForest = isManagedForest;
	}
	
	public String getName() {
		return name;
	}
	
	public boolean isProtectable() {
		return isProtectable;
	}
	
	public boolean isNatural() {
		return isNatural;
	}
	
	public boolean isManagedForest() {
		return isManagedForest;
	}
	
	private static final Map<String, LandCoverType> nameCache = new HashMap<String, LandCoverType>();
    static {
        for (LandCoverType c : values()) {
        	nameCache.put(c.getName(), c);

       }
    }

	public static LandCoverType getForName(String name) {
		LandCoverType type = nameCache.get(name);
		
		if (type == null)
			LogWriter.printlnError("Can't find LandCoverType for " + name);
		
		return type;
	}
	
	
	public static Collection<LandCoverType> getConvertibleTypes() {

		Collection<LandCoverType> convertibleTypes = new HashSet<LandCoverType>();

		for (LandCoverType c : values())
			if (c.isConvertible)
				convertibleTypes.add(c);

		return convertibleTypes;

	}
	
	public static Collection<LandCoverType> getProtectibleTypes() {

		Collection<LandCoverType> protectibleTypes = new HashSet<LandCoverType>();

		for (LandCoverType c : values())
			if (c.isProtectable)
				protectibleTypes.add(c);

		return protectibleTypes;

	}
	
	public static Collection<LandCoverType> getManagedForestTypes() {

		Collection<LandCoverType> managedForestTypes = new HashSet<LandCoverType>();

		for (LandCoverType c : values())
			if (c.isManagedForest)
				managedForestTypes.add(c);

		return managedForestTypes;

	}
}

