package ac.ed.lurg.utils;

import java.util.HashMap;
import java.util.Map;

public class DoubleMap<K, L, V> {
	
	// Stores a double mapped by two consecutive keys
	private Map<K, Map<L, Double>> theMap; 
	
	public DoubleMap() {
		this.theMap = new HashMap<K, Map<L, Double>>(); 
	}
	
	public void put(K key1, L key2, double value) {
		if (theMap.containsKey(key1)) { // check if inner map already exists
			theMap.get(key1).put(key2, value); // adds new entry or overwrites existing if exists
		} else { // add new complete entry
			Map<L, Double> innerMap = new HashMap<L, Double>();
			innerMap.put(key2, value);
			theMap.put(key1, innerMap);
		}
	}

	public Double get(K key1, L key2) {
		if (theMap.containsKey(key1)) { // check if first key exits
			return theMap.get(key1).get(key2); // will return null if second key doesn't exist
		} else {
			return null; // will return null if first key doesn't exist
		}
	}
	
	public Map<L, Double> getInnerMap(K key) {
		return theMap.get(key);

	}
	
	public Map<K, Map<L, Double>> getMap() {
		return theMap;
	}
	
	public void addTo(K key1, L key2, double value) {
		if (theMap.containsKey(key1)) {
			if (theMap.get(key1).containsKey(key2)) {
				double current = get(key1, key2);
				put(key1, key2, current + value);
			} else {
				theMap.get(key1).put(key2, value);
			}
		} else {
			put(key1, key2, value);
		}
	}
}
